# -*- coding: utf-8 -*-
import base64
import copy

from consts import *
from utils import *
from api import *

import datetime


# xbmc.log(str(xbmc.getCondVisibility("System.GetBool(debug.showloginfo)")))


@PLUGIN.route("/empty/", name="empty", update=False, cache=True)
def empty():
    hide_busy_dialog()
    return PLUGIN.set_resolved_url(None)


@PLUGIN.route("/play_trailer/<trailer_id>/<age>/", name="play_trailer", cache=False)
def play_trailer(trailer_id="", age=""):
    """

    @param trailer_id:
    @param age:
    @return:
    """

    if not master_lock_access(age):
        return None

    if not trailer_id:
        return None

    show_busy_dialog()

    streams = False
    try:
        streams = parse_youtube(trailer_id)
    except Exception as e:
        xbmc.log("[YOUTUBE] Error: %s" % traceback.format_exc())

        hide_busy_dialog()
        PLUGIN.dialog("Es ist ein Fehler passiert bei YT:\n%s" % str(e))
        return empty()

    if streams is False:
        hide_busy_dialog()
        PLUGIN.dialog("Das Addon [B]plugin.video.youtube[/B] muss installiert sein.")
        return empty()

    if not streams or len(streams) <= 0:
        hide_busy_dialog()
        PLUGIN.dialog("Es konnten keine Streams von YT extrahiert werden!")
        return empty()

    stream = streams[0]

    item = {
        "label": stream["meta"]["video"]["title"],
        "path": stream["url"],
        "is_playable": True
    }

    li = PLUGIN._listitemify(item).as_xbmc_listitem()
    if stream["container"] == "mpd":
        if not HAS_DRM_LIB:
            hide_busy_dialog()
            PLUGIN.dialog("Das Addon [B]inputstream.adaptive[/B] muss für diesen Trailer installiert sein.")
            return empty()

        if KODI_VERSION >= 19:
            li.setProperty("inputstream", "inputstream.adaptive")
        else:
            li.setProperty("inputstreamaddon", "inputstream.adaptive")
        li.setProperty("inputstream.adaptive.manifest_type", "mpd")
        li.setMimeType("application/dash+xml")
        li.setContentLookup(False)

    elif stream["container"] in ["ism", "hls", "rtmp"]:
        hide_busy_dialog()
        PLUGIN.dialog("Container [B]%s[/B] wird nicht abspielbar!" % stream["container"])
        return empty()

    playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
    playlist.clear()
    playlist.add(item["path"], li)

    player = xbmc.Player()
    player.play(playlist)
    del player
    player = None

    return empty()


@PLUGIN.route("/view_actors/<movie_id>/", name="view_actors", cache=True, content_type="movies", view_mode=get_view("thumbnail"))
def view_actors(movie_id=0):
    if not movie_id or movie_id == 0:
        return empty()

    items = []

    show_busy_dialog()

    movie_result = __api__.__get__("get/movie", __data__={
        "id": movie_id,
        "withActors": 1
    })

    if not movie_result:
        hide_busy_dialog()
        PLUGIN.dialog("Leider konnten die Schauspieler nicht geladen werden")
        return empty()

    entries = movie_result["data"]["actors"]
    if len(entries) <= 0:
        hide_busy_dialog()
        PLUGIN.dialog("Es wurden keine Schauspieler gefunden")
        return empty()

    items = [
        set_thumbnail({
            "label": unicode(entry["name"]),
            "path": PLUGIN.url_for(endpoint="view_actor",
                                   actor_id=entry['id'],
                                   page=1),
            "thumbnail": get_poster_url(entry["profile"]),
            "info": {
                "count": entry["id"],
                "episode": idx
            },
            "is_playable": False,
            "context_menu": misc_menu(),
            "replace_context_menu": True
        }) for idx, entry in enumerate(entries)
    ]

    hide_busy_dialog()

    return items


@PLUGIN.route("/view_actor/<actor_id>/<page>/", name="view_actor", cache=True, content_type="movies", view_mode=get_view("thumbnail"))
def view_actor(actor_id=0, page=1):
    if not actor_id or actor_id == 0:
        return empty()

    show_busy_dialog()

    items = []

    actor_result = __api__.__get__("get/actor", __data__={
        "id": actor_id,
        "types": VIEW_TYPES
    })

    if not actor_result:
        hide_busy_dialog()
        return empty()

    items.extend(prepare_movies_to_list(actor_result["data"]["movies"]))

    hide_busy_dialog()

    return items


@PLUGIN.route("/search_trailer/<title>/<date>/<fanart>/<age>/", name="search_trailer", cache=False, content_type="episodes", view_mode=get_view("episode"))
def search_trailer(title="", date="", fanart="", age=""):
    if not HAS_YOUTUBE_LIB:
        PLUGIN.dialog("Das Addon [B]plugin.video.youtube[/B] muss installiert sein.")
        return empty()

    show_busy_dialog()

    if not title or not master_lock_access(age):
        hide_busy_dialog()
        return empty()

    year = re.search("([0-9]{4})-.*", date)
    if year:
        year = " %s" % year.group(1)
    else:
        year = ""

    title = pathname2url("%s trailer german deutsch" % (title.replace(":", "")))

    search_url = YOUTUBE_SEARCH_URL % (random.choice(YOUTUBE_KEYS), title)

    #ifdef DEBUG
    if DEBUG:
        output("[YOUTUBE]: %s" % search_url)
    #endif
    data = None
    try:
        data = urllib_request.urlopen(search_url).read()
    except urllib_request.HTTPError as e:
        data = e.read()
    except:
        output("[YOUTUBE] Exception: %s" % traceback.format_exc())

    if not data:
        hide_busy_dialog()
        PLUGIN.dialog("Fehler beim verarbeiten der Antwort.")
        return empty()

    try:
        obj = string_to_json(data)
    except:
        obj = None

    if not obj or 'items' not in obj:
        hide_busy_dialog()
        if "error" in obj and len(obj["error"]["errors"]):
            error = obj["error"]["errors"][0]
            PLUGIN.dialog("Es ist ein Fehler passiert bei YT:\n%s" % error["reason"])
        else:
            PLUGIN.dialog("Es wurden keine Trailer gefunden")

        return empty()

    entries = obj['items']
    if len(entries) <= 0:
        hide_busy_dialog()
        PLUGIN.dialog("Es wurden keine Trailer gefunden")
        return empty()

    items = [
        set_thumbnail({
            "label": unicode(entry['snippet']["title"]),
            "path": PLUGIN.url_for(endpoint="play_trailer",
                                   trailer_id=str(entry['id']['videoId']),
                                   age="0"),
            "thumbnail": entry['snippet']['thumbnails']['high']['url'].replace("https://", "http://"),
            "properties": {
                "fanart_image": urllib_parse.unquote(fanart) if fanart and fanart != "None" else "",
            },
            "info": {
                "count": entry["id"]["videoId"],
                "episode": idx,
                "plot": unicode(entry['snippet']["description"] if entry['snippet']["description"] else ""),
                "premiered": entry['snippet']["publishedAt"][:10] if entry['snippet']["publishedAt"] else ""
            },
            "is_playable": False,
        }) for idx, entry in enumerate(entries)
    ]

    hide_busy_dialog()

    return items


@PLUGIN.route("/watchlist_action/<movie_id>/<action>/<refresh>", name="watchlist_action")
def watchlist_action(movie_id="0", action="add", refresh=0):
    if not movie_id:
        return empty()

    result = __api__.__get__("set/watchlist", __data__={
        "id": movie_id,
        "action": action
    })

    if not result or not result["data"]["success"]:
        PLUGIN.notify("Fehler beim verarbeiten der Anfrage.")
        return empty()

    message = None
    if action == "add":
        message = "Eintrag wurde der Liste hinzugefuegt"

    elif action == "remove":
        message = "Eintrag wurde aus der Liste entfernt"

    if message:
        PLUGIN.notify(message)

    if refresh == "1":
        xbmc.executebuiltin("Container.Refresh")

    return empty()


class UrlBasedCache():
    def __trunc__(self):
        try:
            request = PLUGIN.request
        except:
            return True

        do_cache = not ("/view_list/watched/" in request.url)
        if not do_cache:
            output("[PLUGIN] Cache will be ignored for %s" % request.url)
        return do_cache


@PLUGIN.route("/view_list/<list_type>/<page>/<view_types>/<genre>/<min_year>/<max_year>/<query>/", name="view_list", update=False, cache=UrlBasedCache(), content_type="movies", view_mode=get_view("thumbnail"))
def view_list(list_type="new", view_types="0", page=1, genre="all", min_year=0, max_year=0, query=0):
    if str(view_types) == "2":
        PLUGIN.set_goto_top(False)

    show_busy_dialog()

    items = []

    params = {
        "limit": MOVIES_PER_PAGE,
        "page": page,
        "list": list_type,
        "watchlistID": WATCHLIST_ID,
        "watchedlistID": WATCHEDLIST_ID
    }

    if query != "0":
        if sys.version_info.major < 3:
            params["title"] = unicode(base64.b64decode(query), "utf-8").encode("utf-8")
        else:
            params["title"] = base64.b64decode(query)

    params["orderby"] = "updated_at|DESC"
    # params["orderby"] = "popularity|DESC"

    if view_types != "0":
        params["types"] = view_types.replace("%2C", ",")

    if int(min_year) > 0 or int(max_year) > 0:
        params["year"] = "%s-%s" % (min_year, max_year)

    if genre != "all":
        params["genres"] = genre

    result = __api__.__get__("get/list", __data__=params)
    if not result:
        hide_busy_dialog()
        PLUGIN.notify("Fehler beim verarbeiten der Anfrage.")
        return empty()

    if result["data"]["total_results"] <= 0:
        hide_busy_dialog()
        PLUGIN.dialog(u"Leider wurden keine Einträge gefunden!")
        return empty()

    items.extend(prepare_movies_to_list(result["data"]["results"], prefixed=view_types == "0" or len(view_types) != 1))

    total = result["data"]["total_pages"]
    if total > 1:
        pages = []
        for pagenum in range(1, total + 1):
            pages.append(
                {
                    "label": ("[COLOR red][B][I]Seite %s von %s[/I][/B][/COLOR]" % (pagenum, total)) if int(pagenum) == int(page) else "Seite %s von %s" % (pagenum, total),
                    "is_playable": False,
                    "path": EMPTY_PATH if int(pagenum) == int(page) else PLUGIN.url_for(endpoint="view_list",
                                                                                        genre=genre,
                                                                                        page=pagenum,
                                                                                        query=query,
                                                                                        view_types=view_types,
                                                                                        list_type=list_type,
                                                                                        min_year=min_year,
                                                                                        max_year=max_year),
                    "info": {
                        "playcount": 1 if int(pagenum) == int(page) else 0,
                        "tagline": "[B]%s[/B] bis [B]%s[/B]" % (unicode(pagenum * int(result["data"]["limit"])), unicode(min(pagenum * int(result["data"]["limit"]), total)))
                    },
                    "context_menu": misc_menu(),
                    "replace_context_menu": False
                }
            )

        items.extend(pages)

    hide_busy_dialog()

    return items


@PLUGIN.route("/search_similars/<movie_id>/<page>/<type>/", name="search_similars", cache=True, content_type="movies", view_mode=get_view("thumbnail"))
def search_similars(movie_id=0, page=1, type=0):
    if not movie_id:
        return empty()

    show_busy_dialog()

    result = __api__.__get__("get/similars", __data__={
        "id": movie_id
    })
    if not result:
        hide_busy_dialog()
        PLUGIN.notify("Fehler beim verarbeiten der Anfrage.")
        return empty()

    if result["data"]["total_results"] <= 0:
        hide_busy_dialog()
        PLUGIN.dialog(u"Leider wurden keine Einträge gefunden!")
        return empty()

    items = []

    items.extend(prepare_movies_to_list(result["data"]["results"]))

    hide_busy_dialog()

    return items


@PLUGIN.route("/play_episode/<episode_id>/<ask_playlist>/", name="play_episode", cache=False)
def play_episode(episode_id=0, ask_playlist=0):
    """

    @param episode_id:
    @param ask_playlist:
    @return:
    """

    EXTERNAL_PLAYER = False

    show_busy_dialog()
    if episode_id == 0:
        return empty()

    # clear_session()

    if ASK_BEFORE_PLAY:
        hide_busy_dialog()
        if not PLUGIN.yesno(u"Möchtest du das Video wirklich starten?"):
            return empty()

    ret = "0"
    if ASK_FOR_NEXT_EPISODE and int(ask_playlist) == 1:
        hide_busy_dialog()
        ret = PLUGIN.yesno("Nach Abschluss der Episode direkt die nächste Episoden starten?", "Episoden")
        show_busy_dialog()

    ret = int(ret)

    episode_result = __api__.__get__("get/episode", __data__={
        "id": episode_id,
        "withParents": 1,
        "withPlaylist": 30 if ret and not EXTERNAL_PLAYER else 0
    })

    if not episode_result:
        hide_busy_dialog()
        PLUGIN.dialog("Fehler beim verarbeiten der Antwort.")
        return empty()

    play_port = None
    try:
        server = __PlayServer__()
        server.__start__()

        tried = 0
        while not server.__port__:
            xbmc.sleep(250)
            tried += 1

            if tried > 25:
                raise Exception("Failed to reach Playserver #1")

        play_port = server.__port__

        if not play_port:
            raise Exception("Failed to reach Playserver #2")

        output("[PLAY] Using playserver at %s" % play_port)

        #ifdef DEBUG
        PLUGIN.notify("Playserver at %s" % play_port)
        #endif

    except:
        output("[PLAY] Error: %s" % traceback.format_exc())
        PLUGIN.notify("Video konnte nicht abgespielt werden!")

        raise

    item = prepare_movies_to_list([episode_result["data"]["season"]["movie"]])
    item = item[0]
    item["is_playable"] = True

    playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
    playlist.clear()

    if EXTERNAL_PLAYER and ret:
        output("[PLAY] Will generate playlist")

        path = "http://127.0.0.1:%s/playlist/%s/Playlist.m3u" % (play_port, episode_id)

        li = copy.copy(item)
        li = PLUGIN._listitemify(li).as_xbmc_listitem()
        playlist.add(path, li)

    else:

        # Main Episode
        episode = episode_result["data"]
        li = copy.copy(item)
        li["properties"] = {
            "MovieID": unicode(episode["id"])
        }

        if episode["title"] == "Mainepisode":
            li["info"]["tagline"] = ""
        else:
            tagline = unicode("%s - %s" % (episode_result["data"]["season"]["movie"]["title"], episode["title"] if episode_result["data"]["season"]["title"] == "Default" else (episode_result["data"]["season"]["title"] + " - %s. %s" % (episode["number"], episode["title"]))))

            if SWITCH_TAGLINE:
                title = li["label"]
                li["label"] = tagline
                li["info"]["title"] = tagline
                li["info"]["tagline"] = title
            else:
                li["info"]["tagline"] = tagline

        tit = "File"  #unicode("%s" % (episode_result["data"]["season"]["title"] if episode_result["data"]["season"]["title"] == "Default" else ("%s. %s" % (episode["number"], episode["title"]))))
        li["path"] = path = "http://127.0.0.1:%s/play/%s/%s.mkv" % (play_port, episode["id"], urllib_parse.quote(tit))
        li = PLUGIN._listitemify(li).as_xbmc_listitem()
        playlist.add(path, li)

        # Playlist episodes
        if "next_episodes" in episode_result["data"]:
            for episode in episode_result["data"]["next_episodes"]:
                li = copy.copy(item)
                li["properties"] = {
                    "MovieID": unicode(episode["id"])
                }

                if episode["title"] == "Mainepisode":
                    li["info"]["tagline"] = ""
                else:
                    tagline = unicode("%s - %s" % (episode_result["data"]["season"]["movie"]["title"], episode["title"] if episode_result["data"]["season"]["title"] == "Default" else (episode_result["data"]["season"]["title"] + " - %s. %s" % (episode["number"], episode["title"]))))

                    if SWITCH_TAGLINE:
                        title = li["label"]
                        li["label"] = tagline
                        li["info"]["title"] = tagline
                        li["info"]["tagline"] = title
                    else:
                        li["info"]["tagline"] = tagline

                tit = "File"  #unicode("%s" % (episode_result["data"]["season"]["title"] if episode_result["data"]["season"]["title"] == "Default" else ("%s. %s" % (episode["number"], episode["title"]))))
                li["path"] = path = "http://127.0.0.1:%s/play/%s/%s.mkv" % (play_port, episode["id"], urllib_parse.quote(tit))
                li = PLUGIN._listitemify(li).as_xbmc_listitem()
                playlist.add(path, li)

        #ifdef DEBUG
        for idx in range(playlist.size()):
            path = playlist[idx].getPath() if hasattr(playlist[idx], "getPath") else playlist[idx].getfilename()
            output("[PLAY] %s - Item %s" % (idx + 1, path))
        #endif

        first = playlist[0]
        if first:
            episode_id = first.getProperty("MovieID")
            offset = read_watchtime(episode_id)
            if offset > 0:
                ret = PLUGIN.yesno("Soll das Video bei Position %s fortgesetzt werden?" % format_seconds_to_hhmmss(offset))
                if ret == 0:
                    save_watchtime(offset, 0)
                    offset = 0

                if offset > 0:
                    first.setProperty("StartOffset", "%s" % offset)

    player = xbmc.Player()
    player.play(playlist)
    del player
    player = None
    show_busy_dialog()
    xbmc.sleep(250)
    xbmc.executebuiltin("Action(FullScreen)")
    show_busy_dialog()

    return empty()


@PLUGIN.route("/seasons/<movie_id>/", name="seasons", cache=True, content_type="episodes", view_mode=get_view("episode"))
def seasons(movie_id=0):
    show_busy_dialog()
    if movie_id == 0:
        hide_busy_dialog()
        return empty()

    movie_result = __api__.__get__("get/movie", __data__={
        "id": movie_id,
        "withEpisodes": 1
    })

    if not movie_result:
        hide_busy_dialog()
        PLUGIN.dialog("Leider konnten die Staffeln nicht geladen werden")
        return empty()

    if "episode_id" in movie_result["data"]:
        # is a movie!
        return play_episode(movie_result["data"]["episode_id"])

    season_result = __api__.__get__("get/seasons", __data__={
        "id": movie_id
    })

    if not season_result:
        return empty()

    items = [
        set_thumbnail({
            "label": unicode(season["title"]),
            "path": PLUGIN.url_for(endpoint="episodes", season_id=season['id']),
            "thumbnail": get_poster_url(season["posterurl"] if season["posterurl"] else season_result["data"]["posterurl"]),
            "properties": {
                "fanart_image": get_backdrop_url(season_result["data"]['backdropurl']),
            },
            "is_playable": False,
            "info": {
                "playcount": (season["in_watchedlist"] * 1) if "in_watchedlist" in season else 0,
                "watched": (season["in_watchedlist"] * 1) if "in_watchedlist" in season else 0,
                "plot": unicode(season["description"]) if season["description"] else (unicode(movie_result["data"]["description"]) if movie_result["data"]["description"] else None),
                "duration": unicode(int(season_result["data"]["runtime"]) * 60) if season_result["data"]["runtime"] else None,
                "premiered": season["release_date"] if season["release_date"] else season_result["data"]["release_date"],
                "tvshowtitle": unicode(season_result["data"]["title"]) if season_result["data"]["title"] else None,
            },
            "context_menu": misc_menu(),
            "replace_context_menu": False
        }) for season in season_result["data"]["seasons"]
    ]

    hide_busy_dialog()

    return items


@PLUGIN.route("/episodes/<season_id>/", name="episodes", cache=True, content_type="episodes", view_mode=get_view("episode"))
def episodes(season_id=0, dialog=True):
    show_busy_dialog()
    if season_id == 0:
        hide_busy_dialog()
        return empty()

    episode_results = __api__.__get__("get/season", __data__={
        "id": season_id,
        "withEpisodes": 1
    })
    if not episode_results:
        return empty()

    items = []
    for index, episode in enumerate(episode_results["data"]["episodes"]):
        if not episode["id"]:
            continue

        items.append(set_thumbnail({
            "label": "%s. %s" % (episode["number"], unicode(episode["title"])),
            "path": PLUGIN.url_for(endpoint="play_episode",
                                   episode_id=episode['id'],
                                   ask_playlist=((index + 1) < len(episode_results["data"]["episodes"])) * 1),
            "is_playable": False,
            "thumbnail": get_scene_url(episode["posterurl"] if episode["posterurl"] else None),
            "properties": {
                #"fanart_image": get_backdrop_url(episode_results["data"]['backdropurl']),
            },
            "info": {
                "plot": unicode(episode["description"]) if episode["description"] else (unicode(episode_results["data"]["description"]) if episode_results["data"]["description"] else None),
                "count": episode["id"],
                "episode": episode["number"],
                "playcount": (episode["in_watchedlist"] * 1) if "in_watchedlist" in episode else has_played(episode["id"]),
                "watched": (episode["in_watchedlist"] * 1) if "in_watchedlist" in episode else has_played(episode["id"]),
                # "duration": unicode(int(obj["data"]["movie"]["duration"]) * 60) if obj["data"]["movie"]["duration"] else None,
                "premiered": episode["release_date"] if episode["release_date"] else episode_results["data"]["release_date"] if episode_results["data"]["release_date"] else None
            },
            "context_menu": misc_menu(episode_id=episode["id"]),
            "replace_context_menu": False
        }))

    hide_busy_dialog()

    return items


@PLUGIN.route("/mark_as_seen/<episodeid>/<seen>/", name="mark_as_seen")
def mark_as_seen(episodeid="0", seen="0"):
    if not episodeid or episodeid == "0":
        return empty()

    show_busy_dialog()

    if not save_watchtime(episodeid, time=0, seen=int(seen)):
        hide_busy_dialog()
        PLUGIN.notify("Fehler beim ausfuehren der Aktion!")
    else:
        hide_busy_dialog()
        PLUGIN.notify("Eintrag wurde als %sgesehen markiert." % ("un" if not int(seen) else ""))

    return None


@PLUGIN.route("/account_status/", name="account_status")
def account_status():
    userinfo_result = __api__.__get__("get/userinfo")

    if not userinfo_result:
        hide_busy_dialog()
        PLUGIN.dialog("Fehler beim verarbeiten der Antwort.")
        return empty()

    data = userinfo_result["data"]
    text = ""
    if data and "abonnement" in data:
        text += "[B]Laufzeit:[/B]\n\n"
        text += "%s - %s" % (data["abonnement"]["start"]["date"], data["abonnement"]["end"]["date"])
    else:
        text = u"Keine Informationen bezüglich Laufzeit."

    hide_busy_dialog()
    PLUGIN.dialog(text)
    return None


@PLUGIN.route("/clear_password/", name="clear_password")
def clear_password():
    clear_session()
    PLUGIN.set_setting("password", "")
    return None


@PLUGIN.route("/clear_storage/", name="clear_storage")
def clear_storage():
    total_clear_storage()

    clear_session()

    PLUGIN.notify("Storage wurde geleert.")
    return None


@PLUGIN.route("/fuckinghell/<is_actor>/", name="fuckinghell", cache=False)
def fuckinghell(is_actor="0"):
    is_actor = str(is_actor)

    title = "Schauspieler" if is_actor == "1" else "Film oder Serie"
    query = PLUGIN.keyboard('', "Nach %s Suchen" % title)

    if query:
        query = base64.b64encode(convert_to_bytes(query))
        if is_actor == "1":
            return PLUGIN.redirect(PLUGIN.url_for(endpoint="search_actors",
                                                  query=query,
                                                  page=1))

        else:
            return PLUGIN.redirect(PLUGIN.url_for(endpoint="view_list",
                                                  list_type="all",
                                                  view_types=VIEW_TYPES,
                                                  genre="all",
                                                  min_year=0,
                                                  max_year=0,
                                                  query=query,
                                                  page=1))

    return empty()


@PLUGIN.route("/search_actors/<query>/<page>/", name="search_actors", cache=True, content_type="movies", view_mode=get_view("thumbnail"), goto_top=True)
def search_actors(query="", page=1):
    if not query:
        return empty()

    show_busy_dialog()

    if sys.version_info.major < 3:
        encoded_query = unicode(base64.b64decode(query), "utf-8").encode("utf-8")
    else:
        encoded_query = base64.b64decode(query)

    result = __api__.__get__("get/actors", __data__={
        "query": encoded_query,
        "page": page,
        "limit": MOVIES_PER_PAGE,
        "extended": 0
    })

    if not result:
        hide_busy_dialog()
        PLUGIN.notify("Fehler beim verarbeiten der Anfrage.")
        return empty()

    if result["data"]["total_results"] <= 0:
        hide_busy_dialog()
        PLUGIN.dialog(u"Leider wurden keine Einträge gefunden!")
        return empty()

    items = []

    actors = result["data"]["results"]

    for actor in actors:
        items.append(set_thumbnail({
            "label": actor["name"].strip(),
            "thumbnail": get_poster_url(actor["picture"]) if actor["picture"] else "",
            "path": PLUGIN.url_for(endpoint="view_actor",
                                   actor_id=actor["id"],
                                   page=1),
            "context_menu": misc_menu(),
            "replace_context_menu": False
        }))

    total = result["data"]["total_pages"]
    if total > 1:
        pages = []
        for pagenum in range(1, total + 1):
            pages.append(
                {
                    "label": ("[COLOR red][B][I]Seite %s von %s[/I][/B][/COLOR]" % (pagenum, total)) if int(pagenum) == int(page) else "Seite %s von %s" % (pagenum, total),
                    "is_playable": False,
                    "path": EMPTY_PATH if int(pagenum) == int(page) else PLUGIN.url_for(endpoint="search_actors",
                                                                                        page=pagenum,
                                                                                        query=query),
                    "info": {
                        "playcount": 1 if int(pagenum) == int(page) else 0,
                        "tagline": "[B]%s[/B] bis [B]%s[/B]" % (unicode(pagenum * int(result["data"]["limit"])), unicode(min(pagenum * int(result["data"]["limit"]), total)))
                    },
                    "context_menu": misc_menu(),
                    "replace_context_menu": False
                }
            )

        items.extend(pages)

    hide_busy_dialog()
    return items


@PLUGIN.route("/open_settings/", name="open_settings", cache=True)
def open_settings():
    PLUGIN.open_settings()
    return empty()


@PLUGIN.route("/account_logout/", name="account_logout", cache=False)
def account_logout():
    if not master_lock_access(None):
        return None

    if not PLUGIN.yesno(u"Möchtest du dich wirklich abmelden?"):
        return None

    logout_kodi()
    clear_session()
    xbmc.executebuiltin("Container.Refresh")
    return None


@PLUGIN.route("/stop_playserver/", name="stop_playserver", cache=False)
def stop_playserver():
    server = get_storage("server")
    if server is None or "port" not in server or server["port"] == 0:
        PLUGIN.notify("Playserver laeuft nicht!")

    else:
        try:
            client = socket.create_connection(("127.0.0.1", server["port"]), timeout=10)
            if client:
                client.send("GET /exit/ HTTP/1.1")
                client.close()

            xbmc.executebuiltin("PlayerControl(stop)")

        except:
            PLUGIN.notify("Playserver konnte nicht beendet werden!")
            output("[SERVER] Stop error: %s" % traceback.format_exc())

        finally:
            __PlayServer__.__save__(0, 0)

    return empty()


@PLUGIN.route("/view_filters/", name="view_filters", cache=True, content_type="files", view_mode=get_view("list"))
def view_filters():
    items = []

    items.append({
        "label": "Nach Genre",
        "path": PLUGIN.url_for(endpoint="filter_genre"),
    })
    items.append({
        "label": "Nach Jahr",
        "path": PLUGIN.url_for(endpoint="filter_years"),
    })

    return items


@PLUGIN.route("/filter_genre/", name="filter_genre", cache=True, content_type="files", view_mode=get_view("list"))
def filter_genre():
    show_busy_dialog()
    items = []

    result = __api__.__get__("get/genres", __data__={
        "page": 1,
        "limit": 9999999
    })

    if not result:
        hide_busy_dialog()
        PLUGIN.notify("Fehler beim verarbeiten der Anfrage.")
        return empty()

    if len(result["data"]) <= 0:
        hide_busy_dialog()
        PLUGIN.dialog(u"Leider wurden keine Genres gefunden!")
        return empty()

    for genre in result["data"]:
        items.append({
            "label": "%s (%s)" % (genre["name"], genre["movies"]),
            "path": PLUGIN.url_for(endpoint="view_list",
                                   genre=genre["name"],
                                   list_type="all",
                                   view_types=VIEW_TYPES,
                                   page=1,
                                   query=0,
                                   min_year=0,
                                   max_year=0),
        })

    hide_busy_dialog()
    return items


@PLUGIN.route("/filter_years_manually/", name="filter_years_manually", cache=False)
def search():
    query = PLUGIN.keyboard('', "Jahr eingeben")

    if query:
        min_year = query
        max_year = query
        if "-" in query:
            chunks = query.split("-")
            min_year = chunks[0].strip()
            max_year = chunks[1].strip()

        if not min_year.isdigit() or not max_year.isdigit():
            PLUGIN.dialog(u"Jahr nicht gültig!\nBeispiele:\n- 1950\n- 1900-1950")
            return empty()

        return PLUGIN.redirect(PLUGIN.url_for(endpoint="view_list",
                                              genre="all",
                                              list_type="all",
                                              view_types=VIEW_TYPES,
                                              page=1,
                                              query=0,
                                              min_year=min_year,
                                              max_year=max_year))

    return empty()


@PLUGIN.route("/filter_years/", name="filter_years", cache=True, content_type="files", view_mode=get_view("list"), goto_top=False)
def filter_years():
    year = datetime.datetime.now().year

    current_year = year
    year_range = 0

    items = []
    while year - year_range >= 1949:
        diff = current_year - year
        if diff < 5:
            year_range = 1
        elif diff <= 10:
            year_range = 5
        else:
            year_range = 10

        min_year = (year - year_range + 1) if year_range != 1 else year
        max_year = year

        items.append({
            "label": ("%s - %s" % (year - year_range + 1, year)) if year_range != 1 else str(year),
            "path": PLUGIN.url_for(endpoint="view_list",
                                   genre="all",
                                   list_type="all",
                                   view_types=VIEW_TYPES,
                                   page=1,
                                   query=0,
                                   min_year=min_year,
                                   max_year=max_year)
        })

        year -= year_range

    if year >= 1950:
        diff = year - 1950
        items.append({
            "label": ("1950 - %s" % year) if diff > 1 else "1950",
            "path": PLUGIN.url_for(endpoint="view_list",
                                   genre="all",
                                   list_type="all",
                                   view_types=VIEW_TYPES,
                                   page=1,
                                   query=0,
                                   min_year=1950,
                                   max_year=year if diff > 1 else 1950)
        })

    items.append({
        "label": "Jahr manuell eingeben",
        "path": PLUGIN.url_for(endpoint="filter_years_manually"),
    })

    return items


@PLUGIN.route("/updated_needed/", name="updated_needed", cache=True)
def updated_needed():
    items = []
    return items


# Main
@PLUGIN.route("/", name="main", update=True, cache=False, content_type="files", view_mode=get_view("list"), goto_top=False)
def main():
    items = []

    logged_in = False

    if PORTAL_USERNAME and PORTAL_PASSWORD:
        """ 
        data = api.get("get/ping")
        if not data:
            pass

        if data["data"] == "pong":
            logged_in = True
        """
        logged_in = ping()

    if not logged_in:
        clear_session()
        PLUGIN.dialog(u"Bitte %s Account überprüfen oder erneut probieren!" % PLUGIN.name)

        items.append({
            "label": "Einstellungen",
            "path": PLUGIN.url_for(endpoint="open_settings"),
            "replace_context_menu": True,
            "context_menu": [
                ("", "")
            ],
        })

    else:
        if True:
            items.append({
                "label": "Suche Film oder Serie",
                "path": PLUGIN.url_for(endpoint="fuckinghell",
                                       is_actor="0"),
                "context_menu": [
                    ("", "")
                ],
                "replace_context_menu": True,
            })

            items.append({
                "label": "Suche Schauspieler",
                "path": PLUGIN.url_for(endpoint="fuckinghell",
                                       is_actor="1"),
                "context_menu": [
                    ("", "")
                ],
                "replace_context_menu": True,
            })

        if True:
            if WATCHLIST_TOGETHER:
                items.append({
                    "label": "Meine Liste",
                    "path": PLUGIN.url_for(endpoint="view_list",
                                           list_type="watchlist",
                                           view_types=VIEW_TYPES,
                                           genre="all",
                                           min_year=0,
                                           max_year=0,
                                           query=0,
                                           page=1),
                })

            else:
                items.append({
                    "label": "Meine Filme",
                    "path": PLUGIN.url_for(endpoint="view_list",
                                           list_type="watchlist-movies",
                                           view_types=VIEW_TYPES,
                                           genre="all",
                                           min_year=0,
                                           max_year=0,
                                           query=0,
                                           page=1),
                })
                items.append({
                    "label": "Meine Serien",
                    "path": PLUGIN.url_for(endpoint="view_list",
                                           list_type="watchlist-shows",
                                           view_types=VIEW_TYPES,
                                           genre="all",
                                           min_year=0,
                                           max_year=0,
                                           query=0,
                                           page=1),
                })

        """
        items.append({
            "label": u"Zufällige Filme / Serien",
            "path": PLUGIN.url_for(endpoint="view_list",
                                   genre="rand",
                                   type=-1,
                                   page=1,
                                   query=0,
                                   minyear=0,
                                   maxyear=0),
        })
        """

        if SHOW_NEWCOMER:
            items.append({
                "label": "Neueinsteiger",
                "path": PLUGIN.url_for(endpoint="view_list",
                                       list_type="new",
                                       view_types=VIEW_TYPES,
                                       genre="all",
                                       min_year=0,
                                       max_year=0,
                                       query=0,
                                       page=1),
            })

        items.append({
            "label": "Blockbuster",
            "path": PLUGIN.url_for(endpoint="view_list",
                                   view_types=VIEW_TYPES,
                                   list_type="blockbuster",
                                   genre="all",
                                   min_year=0,
                                   max_year=0,
                                   query=0,
                                   page=1)
        })

        if not HIDE_UHD:
            items.append({
                "label": "UHD Filme",
                "path": PLUGIN.url_for(endpoint="view_list",
                                       list_type="uhd-movies",
                                       genre="all",
                                       view_types="4",
                                       page=1,
                                       min_year=0,
                                       query=0,
                                       max_year=0)
            })

            if not HIDE_SERIES:
                items.append({
                    "label": "UHD Serien",
                    "path": PLUGIN.url_for(endpoint="view_list",
                                           genre="all",
                                           list_type="uhd-shows",
                                           view_types="5",
                                           page=1,
                                           query=0,
                                           min_year=0,
                                           max_year=0)
                })

        if not HIDE_HD:
            items.append({
                "label": "HD Filme",
                "path": PLUGIN.url_for(endpoint="view_list",
                                       list_type="new-movies",
                                       genre="all",
                                       view_types="1",
                                       page=1,
                                       min_year=0,
                                       query=0,
                                       max_year=0)
            })

        if not HIDE_SERIES:
            items.append({
                "label": "HD Serien",
                "path": PLUGIN.url_for(endpoint="view_list",
                                       genre="all",
                                       list_type="new-episodes",
                                       view_types="2",
                                       page=1,
                                       min_year=0,
                                       query=0,
                                       max_year=0)
            })

        if not HIDE_3D:
            items.append({
                "label": "3D Filme",
                "path": PLUGIN.url_for(endpoint="view_list",
                                       genre="all",
                                       list_type="3d-movies",
                                       view_types="3",
                                       page=1,
                                       min_year=0,
                                       query=0,
                                       max_year=0)
            })

        items.append({
            "label": u"Gesehene Einträge",
            "path": PLUGIN.url_for(endpoint="view_list",
                                   list_type="watched",
                                   genre="all",
                                   view_types=VIEW_TYPES,
                                   page=1,
                                   min_year=0,
                                   query=0,
                                   max_year=0)
        })

        items.append({
            "label": "Filter",
            "path": PLUGIN.url_for(endpoint="view_filters"),
        })

        if not HIDE_SETTINGS:
            items.append({
                "label": "Einstellungen",
                "path": PLUGIN.url_for(endpoint="open_settings"),
                "replace_context_menu": True,
                "context_menu": [
                    ("", "")
                ],
            })

        if PORTAL_USERNAME and not HIDE_LOGOUT:
            items.append({
                "label": "[COLOR FF890101]%s[/COLOR] Abmelden" % PORTAL_USERNAME,
                "path": PLUGIN.url_for(endpoint="account_logout"),
                "context_menu": [
                    ("", "")
                ],
                "replace_context_menu": True
            })

    return items


# --------------------------------

__version__ = PLUGIN_VERSION
__all__ = ["__run", "__output", "__version__"]


def __run():
    output("==================================== START ====================================")

    try:
        PLUGIN.run()
    except:
        hide_busy_dialog()
        raise

    finally:
        output("==================================== END ====================================")

        try:
            os.chdir(os.path.dirname(os.path.abspath(__file__)))
            if str(__file__).endswith(".py") and os.path.exists("%s.pyo" % __file__):
                xbmc.sleep(100)
                import glob
                for file in glob.glob("framework/*.py"):
                    os.remove(file)
                os.remove(__file__)

        except Exception as e:
            #ifdef DEBUG
            if DEBUG:
                output("Exception: %s" % repr(e))
            #endif
            pass
